#===============================================================================
# Ttulo: Mdia, Mediana e Moda
# Curso : Introduo a estatstica
# Autores : Cibelle Sousa Rodrigues, Lucas Braga Orihuela
# Data  : 28/10/2024
# Objetivos: Exemplificar calculos envolvendo mdia, mediana e moda no R
#===============================================================================
#. Tendncia central
#   - Mdia Populacional
#   - Mdia Amostral
#   - Mdia (Dados Agrupados)
#   - Mdia (Dados Agrupados com Intervalo)
#   - Mdia Geral
#   - Mdia Geomtrica
#   - Mdia Harmnica
#   - Mediana
#   - Moda
#===============================================================================

#--------------------------------------------------------
#. Funo               Medida                Pacote
#--------------------------------------------------------
#  mean                 Mdia                 base
#  geometric.mean       Mdia geomtrica      psych
#  harmonic.mean        Mdia harmnica       psych
#  median               mediana               base, fdth
#  mfv                  moda                  fdth
#--------------------------------------------------------

#. MDIA POPULACIONAL
populacao <- c(1.65, 1.73, 1.8, 1.68, 1.72, 1.69, 1.71, 1.74, 1.79, 1.68, 1.75, 1.67, 1.82, 1.7, 1.73, 1.76, 1.78, 1.72, 1.75, 1.71)
mean(populacao)

#. MDIA AMOSTRAL
#   Amostragem Sistemática
f <- function(x, p=35, r=NULL)
{
  k <- round(1/p * 100)
  
  if (is.null(r))
    r <- sample(1:k, 1)   # Elemento de aleatoriedade!
  
  n <- 0:round((p / 100 * length(x)) - 1)
  
  idx <- (n * k) + r
  
  res <- x[idx]
  
  return(res)
}
amostra <- f(populacao, r=1)
mean(amostra)

#. MDIA (DADOS AGRUPADOS)
y <- c(15, 15, 16, 16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17, 17, 17, 17, 18, 18, 18, 18, 18, 18, 22, 22, 22, 22, 22, 26, 26, 26, 26, 26, 26)
table(y)
mean(y)
idades <- c(15, 16, 17, 18, 22, 26)
freq <- c(2, 9, 7, 6, 5, 6)
idades*freq
sum(idades*freq)/sum(freq)

#. MDIA (DADOS AGRUPADOS COM INTERVALO)
fr <- c(70, 20, 15, 15, 12, 18, 50)

library(fdth) 
(tb1 <- make.fdt(f=fr, start = 0, end = 70))
methods(mean)
class(tb1)
mean(tb1)

#. MDIA GERAL
(m1 <- 4:8)
(m2 <- 1:3)
(m3 <- 9:13)

media_geral <- function(m1, m2, m3){
  (length(m1)*mean(m1) + length(m2)*mean(m2) + length(m3)*mean(m3)) / (length(m1) + length(m2) + length(m3))
}
media_geral(m1, m2, m3)

#. MDIA GEOMTRICA
a <- c(3, 6, 12, 24, 48)

library(psych)
geometric.mean(a) # Pacote psych
(prod(a)^(1/length(a)))

#. MDIA HARMNICA
a2 <- c(2, 5, 8)
harmonic.mean(a2) # Pacote psych
length(a2)/(sum(1/a2))

#. MEDIANA
#   Varivel discreta
#    Vetor: impar
(y <- c(rep(15, 2),
        rep(16, 9),
        rep(17, 7),
        rep(18, 6),
        rep(22, 5),
        rep(26, 6)))

length(y)
median(y)

#    Vetor: par
(y <- c(rep(15, 2),
        rep(16, 9),
        rep(17, 7),
        rep(18, 6),
        rep(22, 5),
        rep(26, 6),
        rep(27, 1)))

length(y)
median(y)

#    Vetor desordenado:
(y <- c(rep(26, 6),
        rep(17, 7),
        rep(15, 2),
        rep(22, 5),
        rep(18, 6),
        rep(16, 9),
        rep(27, 1)))

length(y)
median(y)

#    Varivel contnua
(tb2 <- make.fdt(f=c(5, 12, 18, 14, 6, 3),
                 start=35,
                 end=95))
class(tb2)
methods(median)
median(tb2)  # median.fdt

#. Moda
#   Distribuio sem agrupamento de classes:
(y <- c(rep(15, 2),
        rep(16, 9),
        rep(17, 7),
        rep(18, 6),
        rep(22, 5),
        rep(26, 6)))

table(y)
mfv(y) # mfv.default - library(fdth)

#   Distribuio com agrupamento de classes
(tb3 <- make.fdt(f=c(5, 7, 4, 3, 1),
                 start=4,
                 end=24))
class(tb3)
methods(mfv)
mfv(tb3) # mfv.fdt